#!/usr/bin/env python
# coding=utf-8

from distutils.core import setup
from distutils.extension import Extension
from Pyrex.Distutils import build_ext
from distutils.command.install import install as _install
import os, re, sys

from DistUtilsExtra.command import *

INSTALLED_FILES = "installed_files"

def setvar(file, var, value, quotes=True):
	lines = ""
	found = False
	try:
		f = open(file, "r")
	except:
		f = open(file, "w")
		f.close()
		f = open(file, "r")
	
	for line in f:
		if re.match("^"+var, line):
			if quotes:
				lines += var + "=" + '"' + value + '"\n'
			else:
				lines += var + "=" +  value + '\n'
			found = True
		else:
			lines += line
			
	if not found:
		if quotes:
			lines += var + "=" + '"' + value + '"\n'
		else:
			lines += var + "=" +  value + '\n'		
		
			
	f.close()
	f = open(file, "w")
	f.write(lines)
	f.close()
	
def getvar(file, var):

	try:
		f = open(file, "r")
	except:
		return
	
	for line in f:
		if re.match("^"+var, line):
			return line.split("=")[1].strip().strip("\"")
	
	f.close()

#ops = ("install", "build", "sdist", "uninstall", "clean", "build_i18n")

#if len (sys.argv) < 2 or sys.argv[1] not in ops:
#    print "Please specify operation : %s" % " | ".join (ops)
 #   raise SystemExit
    
VERSION = getvar("src/blueman/constants.py", "VERSION")


prefix = None
if len (sys.argv) > 2:
    i = 0
    for o in sys.argv:
        if o.startswith ("--prefix"):
            if o == "--prefix":
                if len (sys.argv) >= i:
                    prefix = sys.argv[i + 1]
                sys.argv.remove (prefix)
            elif o.startswith ("--prefix=") and len (o[9:]):
                prefix = o[9:]
            sys.argv.remove (o)
            break
        i += 1
if not prefix and "PREFIX" in os.environ:
    prefix = os.environ["PREFIX"]
if not prefix or not len (prefix):
    prefix = "/usr"

if sys.argv[1] in ("install", "uninstall") and len (prefix):
    sys.argv += ["--prefix", prefix]
    
setvar("src/blueman/constants.py", "PREFIX", prefix)


class install (_install):

    def run (self):
        _install.run (self)
        outputs = self.get_outputs ()
        length = 0
        if self.root:
            length += len (self.root)
        if self.prefix:
            length += len (self.prefix)
        if length:
            for counter in xrange (len (outputs)):
                outputs[counter] = outputs[counter][length:]
        data = "\n".join (outputs)
        try:
            file = open (INSTALLED_FILES, "w")
        except:
            self.warn ("Could not write installed files list %s" % \
                       INSTALLED_FILES)
            return 
        file.write (data)
        file.close ()

class uninstall (_install):

    def run (self):
        try:
            file = open (INSTALLED_FILES, "r")
        except:
            self.warn ("Could not read installed files list %s" % \
                       INSTALLED_FILES)
            return 
        files = file.readlines ()
        file.close ()
        prepend = ""
        if self.root:
            prepend += self.root
        if self.prefix:
            prepend += self.prefix
        if len (prepend):
            for counter in xrange (len (files)):
                files[counter] = prepend + files[counter].rstrip ()
        for file in files:
            print "Uninstalling %s" % file
            try:
                os.unlink (file)
            except:
                self.warn ("Could not remove file %s" % file)


def get_files(prefix, _dir, match, init_dir = None):
	if init_dir == None:
		init_dir = _dir
	
	list = []
	files = []
	dir = os.listdir(_dir)
	for i in range(len(dir)):

		if os.path.isdir(_dir + "/" + dir[i]):
			#print _dir + "/" + dir[i]
			list += get_files(prefix, _dir + "/" + dir[i], match, init_dir)

		else:
			if re.search(match, dir[i]):
				files.append(_dir + "/" + dir[i])
	if len(files) > 0:		
		list.append((prefix + _dir.replace(init_dir, ""), files))
			
	return list 

setup(
	name='blueman',
      	version=VERSION,
      	author="Valmantas Palikša",
      	author_email="walmis@balticum-tv.lt",
      	url="blueman.tuxfamily.org",
      	description="A graphical bluetooth manager",
      	license='GPL',
      	packages=['blueman', 'blueman.Gui', 'blueman.Main', 'blueman.Services', 'blueman.Bluetooth', 'blueman.Services.gui', 'blueman.Services.bluetooth', 'blueman.Services.bluetooth.Transfer', 'blueman.Services.service_manager']
,
  
      	package_dir = {'blueman': 'src/blueman'},
      	scripts=['src/script/blueman'],
      	ext_package='blueman',
        ext_modules=[ 
   	Extension("blueman", ["module/blueman.pyx", "module/src/libblueman.c"], libraries = ["bluetooth"], include_dirs=['module/include'])
   	],

  	data_files= get_files('share/blueman', "src/glade", ".*\.glade$") +
                    get_files('share/blueman/icons', "icons", ".*\.(png|svg)$") +
		    [('share/icons/hicolor/48x48/apps', ["icons_sys/blueman.png"]),
		    ('share/icons/hicolor/scalable/apps', ["icons_sys/blueman.svg"]),
		    ('share/icons/hicolor/scalable/devices', ["icons_sys/bluetooth-device.svg"]),
		    ('/usr/share/dbus-1/system-services', ["sys/org.blueman.Mechanism.service"]),
		    ('/etc/dbus-1/system.d', ["sys/org.blueman.Mechanism.conf"]),
		    ('/usr/lib/blueman', ["src/script/blueman-mechanism"]),
		    ('/etc/bluetooth', ["sys/network.conf"]),
		    ('/usr/bin', ["sys/blueman-ifup"])
		    ],

        cmdclass = {"build" : build_extra.build_extra,
                    "build_i18n" :  build_i18n.build_i18n,
       		    "install" : install, 
        	    "uninstall" : uninstall,
        	    "build_ext": build_ext
                   }
                   
)

