#
#
# bond_dialog.py
# (c) 2007 Valmantas Paliksa <walmis at balticum-tv dot lt>
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
# 
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# You should have received a copy of the GNU Lesser General Public
# License along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#

import blueman.Globals as Globals
import gtk
import gobject
import dbus

from blueman.Gui.generic_dialog import generic_dialog

_ = Globals._


class bond_dialog(generic_dialog):
	
	def __init__(self, ok, err, addr, name):
		generic_dialog.__init__(self, "bonding.glade", "bond_dialog")
		
		self.ok = ok
		self.err = err
		
		self.response = None
		
		self.xml.signal_autoconnect(self)
		
		self.toggle = self.xml.get_widget("checkbutton1");
		self.text = self.xml.get_widget("entry1");
		
		self.name = self.xml.get_widget("name");
		self.addr = self.xml.get_widget("address");
		
		self.okbutton = self.xml.get_widget("okbutton1");
		
		self.name.set_label("<b>" + name + "</b>")
		self.addr.set_label("<i>" + addr + "</i>")
		
		image = self.xml.get_widget("image1");
		image.set_from_file(Globals.IMAGE_DIR + "/hasbonding.png")
		
		self.dialog.show()
		self.gsource = gobject.timeout_add(15000, self.timeout)
		
	def timeout(self):
		self.err(dbus.DBusException("org.bluez.Error.Canceled"))
		Globals.dbg.warn("Bonding has timed out", 5)
		self.destroy()
		return False	
		
	def on_input_toggled(self, checkbox):
		if self.toggle.get_active():
			self.text.set_visibility(True)
		else:
			self.text.set_visibility(False)
			
	def on_passkey_changed(self, widget):
		if len(widget.get_text()) > 0:
			self.okbutton.set_sensitive(True)
		else:
			
			self.okbutton.set_sensitive(False)
			
			
	def destroy(self):
		self.ok = None
		self.err = None
		gobject.source_remove(self.gsource)
		self.__destroy__()
		
	def __dialog_response__(self, dialog, res):

		if res == gtk.RESPONSE_OK:
			Globals.Statusbar.push(_("Passkey sent. Waiting for confirmation"))
			self.ok(self.get_passkey())
			self.destroy()

		else:
			self.err(dbus.DBusException("org.bluez.Error.Rejected"))
			self.destroy()

		
	def get_passkey(self):
		return self.text.get_text()

