#
#
# marshaller.py - saves internal data structures to files
# (c) 2008 Valmantas Paliksa <walmis at balticum-tv dot lt>
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
# 
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# You should have received a copy of the GNU Lesser General Public
# License along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#


import marshal
import blueman.Globals as Globals
import threading

lock = threading.Lock()
class marshaller:
	cache = {}
	def __init__(self, filepath):
		
		
		self.filepath = filepath
		data = None
		
		lock.acquire()
		if not filepath in marshaller.cache:
		
			try:
				f = open(filepath, "rb")
				data = marshal.load(f)
				Globals.dbg.info("Marshaller: loading file: %s" % filepath, 7)
				marshaller.cache[filepath] = data
				f.close()
			except:
				marshaller.cache[filepath] = None
		lock.release()
		
	def __getattr__(self, name):
		Globals.dbg.info("marshaller::__getattr__(%s)" % (name), 7)
		if name == "data":
			lock.acquire()
			data = marshaller.cache[self.filepath]
			lock.release()
			return data

		else:
			return self.__dict__[name]
			
	def __setattr__(self, name, value):
		lock.acquire()
		Globals.dbg.info("marshaller::__setattr__(%s,%s)" % (name, value), 7)
		if name == "data":
			
			marshaller.cache[self.filepath] = value
			
		else:
			self.__dict__[name] = value
		lock.release()
		
	def save(self):
		Globals.dbg.info("marshaller::save()", 7)
		
		if not self.filepath in marshaller.cache:
			self.__init__(self.filepath)
		
		lock.acquire()	
		f = open(self.filepath, "wb")
		marshal.dump(marshaller.cache[self.filepath], f)
		f.close()
		lock.release()
		
	def free(self):
		lock.acquire()
		Globals.dbg.info("marshaller::free()", 7)
		self.save()
		del marshaller.cache[self.filepath]
		lock.release()
