#
#
# bluetooth.serial.py - BlueZ serial service implementation
# (c) 2007 Valmantas Paliksa <walmis at balticum-tv dot lt>
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
# 
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# You should have received a copy of the GNU Lesser General Public
# License along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#

import blueman.Globals as Globals
import gobject
import dbus
import os

import marshal

from blueman.Services.bluetooth.generic import generic as generic_service
from blueman.Main.mechanism import mechanism
from blueman.Main.marshaller import marshaller

_ = Globals._

class serial(generic_service):
	__gtype_name__ = 'serial_service'
	
	__gsignals__ = {
		'port-created' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, (gobject.TYPE_STRING,)),
		'port-removed' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, (gobject.TYPE_STRING,)),
		'proxy-created' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, (gobject.TYPE_STRING,)),
		'proxy-removed' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, (gobject.TYPE_STRING,))

	}
	
	def __init__(self):
		generic_service.__init__(self, "serial")
		
		self.dun_ports = marshaller(os.path.expanduser("~/.blueman/dun_ports.dat"))
		if self.dun_ports.data == None:
			self.dun_ports.data = []
			
		if self.service.IsRunning():
			self.hal_load_modem_ports()
			
		self.connect("service-started", self.hal_load_modem_ports)
		self.connect("service-stopped", self.hal_unload_modem_ports)


	def hal_unload_modem_ports(self, *args):
		for x in self.dun_ports.data:
			self.hal_uregister_modem_port(x[1], x[0], False)			
		
	def hal_load_modem_ports(self, *args):
		for x in self.dun_ports.data:
			self.hal_register_modem_port(x[1])
		
	def modem_isset(self, path):
		for x in self.dun_ports.data:
			if path in x:
				return True
				
		return False

		
	def hal_register_modem_port(self, path):
		Globals.dbg.info("Registering rfcomm device with HAL [%s]" % path, 5)
		port = self.get_port(path)
		
		dev = port.GetDevice()
		
		if not self.modem_isset(path):
			self.dun_ports.data.append((dev,path))
			self.dun_ports.save()
		
		m = mechanism()
		m.HalRegisterModemPort(dev, port.GetAddress())
		
	def hal_uregister_modem_port(self, path, dev=None, remove=True):
		Globals.dbg.info("Unregistering rfcomm device from HAL [%s]" % path, 5)
		if dev == None:
			
			port = self.get_port(path)
		
			dev = port.GetDevice()
			
		if remove:
			self.dun_ports.data.remove((dev,path))
			self.dun_ports.save()
		
		m = mechanism()
		m.HalUnregisterModemPort(dev)

		
	def connect_signals(self):
	
		self.add_signal(self.port_created, 'PortCreated', 'org.bluez.serial.Manager', None, None)
		self.add_signal(self.port_removed, 'PortRemoved', 'org.bluez.serial.Manager', None, None)
		self.add_signal(self.proxy_created, 'ProxyCreated', 'org.bluez.serial.Manager', None, None)
		self.add_signal(self.proxy_removed, 'ProxyRemoved', 'org.bluez.serial.Manager', None, None)
		#self.add_signal(self.service_connected, 'ServiceConnected', 'org.bluez.serial.Port', None, None, path_keyword="path")
		#self.add_signal(self.service_disconnected, 'ServiceDisconnected', 'org.bluez.serial.Port', None, None, path_keyword="path")
		
	def port_created(self, path):
		self.emit("port-created", path)
		
	def port_removed(self, path):
		self.emit("port-removed", path)
		
	def proxy_created(self, path):
		self.emit("proxy-created", path)
		
	def proxy_removed(self, path):
		self.emit("proxy-removed", path)
		
#	def service_connected(self, path):
#		pass
#		
#	def service_disconnected(self, path):
#		pass

	def create_port(self, address, pattern, **kwargs):
		return self.service_manager.CreatePort(address, pattern, **kwargs);


    	def get_ports(self):

    		ports = self.service_manager.ListPorts()
    		ret = []
    		for i in range(len(ports)):
        		port = self.get_port(ports[i])
        		print port.GetInfo()
        		ret.append((port, ports[i]))
        
    		return ret
    		
    	def get_proxies(self):

    		ports = self.service_manager.ListProxies()
    		ret = []
    		for i in range(len(ports)):
        		port = self.get_proxy(ports[i])
        		ret.append((port, ports[i]))
       		return ret
    		
    	def get_port(self, path):
    		return self.get_interface(path, 'org.bluez.serial.Port')
    		
    	def get_proxy(self, path):
    		return self.get_interface(path, 'org.bluez.serial.Proxy')
    	
    	def get_interface(self, path, interface):
	        obj = self.bus.get_object(self.service_bus, path)
		iface = dbus.Interface(obj, interface)
		return iface

